@@ -19,10 +19,6 @@
 #ifndef _LIBSMB2_H_
 #define _LIBSMB2_H_
 
-#ifndef UINT64_MAX
-#include <stdint.h>
-#endif
-
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -32,7 +28,7 @@ struct smb2_iovec {
         size_t len;
         void (*free)(void *);
 };
-        
+
 struct smb2_context;
 
 /*
@@ -45,33 +41,34 @@ typedef void (*smb2_command_cb)(struct smb2_context *smb2, int status,
 /* Stat structure */
 #define SMB2_TYPE_FILE      0x00000000
 #define SMB2_TYPE_DIRECTORY 0x00000001
+#define SMB2_TYPE_LINK      0x00000002
 struct smb2_stat_64 {
         uint32_t smb2_type;
         uint32_t smb2_nlink;
         uint64_t smb2_ino;
         uint64_t smb2_size;
-	uint64_t smb2_atime;
-	uint64_t smb2_atime_nsec;
-	uint64_t smb2_mtime;
-	uint64_t smb2_mtime_nsec;
-	uint64_t smb2_ctime;
-	uint64_t smb2_ctime_nsec;
+        uint64_t smb2_atime;
+        uint64_t smb2_atime_nsec;
+        uint64_t smb2_mtime;
+        uint64_t smb2_mtime_nsec;
+        uint64_t smb2_ctime;
+        uint64_t smb2_ctime_nsec;
     uint64_t smb2_btime;
     uint64_t smb2_btime_nsec;
 };
 
 struct smb2_statvfs {
-	uint32_t	f_bsize;
-	uint32_t	f_frsize;
-	uint64_t	f_blocks;
-	uint64_t	f_bfree;
-	uint64_t	f_bavail;
-	uint32_t	f_files;
-	uint32_t	f_ffree;
-	uint32_t	f_favail;
-	uint32_t	f_fsid;
-	uint32_t	f_flag;
-	uint32_t	f_namemax;
+        uint32_t        f_bsize;
+        uint32_t        f_frsize;
+        uint64_t        f_blocks;
+        uint64_t        f_bfree;
+        uint64_t        f_bavail;
+        uint32_t        f_files;
+        uint32_t        f_ffree;
+        uint32_t        f_favail;
+        uint32_t        f_fsid;
+        uint32_t        f_flag;
+        uint32_t        f_namemax;
 };
 
 struct smb2dirent {
@@ -89,8 +86,8 @@ typedef int t_socket;
 /*
  * Create an SMB2 context.
  * Function returns
- *  NULL : Failed to create a context.
- *  *nfs : A pointer to an smb2 context.
+ *  NULL  : Failed to create a context.
+ *  *smb2 : A pointer to an smb2 context.
  */
 struct smb2_context *smb2_init_context(void);
 
@@ -106,8 +103,22 @@ struct smb2_context *smb2_init_context(void);
 void smb2_destroy_context(struct smb2_context *smb2);
 
 /*
- * The following three functions are used to integrate libsmb2 in an event
+ * EVENT SYSTEM INTEGRATION
+ * ========================
+ * The following functions are used to integrate libsmb2 in an event
  * system.
+ *
+ * The simplest way is by using smb2_get_fd() and smb2_which_events()
+ * in every loop of the event system to detect which fd to use (it can change)
+ * and which events should be waited for.
+ * This is very simple to use but has the drawback of the overhead having to
+ * call these two functions for every loop.
+ *
+ * This is suitable for trivial apps where you roll your event system
+ * using select() or poll().
+ *
+ * See for example smb2-cat-async.c for an example on how to use these
+ * two functions in an event loop.
  */
 /*
  * Returns the file descriptor that libsmb2 uses.
@@ -117,6 +128,44 @@ t_socket smb2_get_fd(struct smb2_context *smb2);
  * Returns which events that we need to poll for for the smb2 file descriptor.
  */
 int smb2_which_events(struct smb2_context *smb2);
+
+/*
+ * Returns file descriptors that libsmb2 use or is trying to connect to
+ *
+ * This function should be used when trying to connect with more than one
+ * addresses in parallel, cf. rfc8305: Happy Eyeballs.
+ *
+ * The timeout, in ms, is valid during the socket connection step. The caller
+ * should call smb2_service_fd() with fd = -1 when the timeout is reached.
+ * This will trigger a new socket connection on the next resolved address. All
+ * connecting fds will be closed when the first fd is connected. The timeout
+ * will be -1 (infinite) once connected or if there is no next addresses to
+ * connect to.
+ */
+const t_socket *
+smb2_get_fds(struct smb2_context *smb2, size_t *fd_count, int *timeout);
+
+/*
+ * A much more scalable way to use smb2_fd_event_callbacks() to register
+ * callbacks for libsmb2 to call anytime a filedescriptor is changed or when
+ * the  events we are waiting for changes.
+ * This way libsmb2 will do callbacks back into the application to inform
+ * when fd or events change.
+ *
+ * This is suitable when you want to plug libsmb2 into a more sophisticated
+ * eventsystem or if you use epoll() or similar.
+ *
+ * See for smb2-ls-async.c for a trivial example of using these callbacks.
+ */
+#define SMB2_ADD_FD 0
+#define SMB2_DEL_FD 1
+typedef void (*smb2_change_fd_cb)(struct smb2_context *smb2, int fd, int cmd);
+typedef void (*smb2_change_events_cb)(struct smb2_context *smb2, int fd,
+                                      int events);
+void smb2_fd_event_callbacks(struct smb2_context *smb2,
+                             smb2_change_fd_cb change_fd,
+                             smb2_change_events_cb change_events);
+
 /*
  * Called to process the events when events become available for the smb2
  * file descriptor.
@@ -129,6 +178,51 @@ int smb2_which_events(struct smb2_context *smb2);
  */
 int smb2_service(struct smb2_context *smb2, int revents);
 
+/*
+ * Called to process the events when events become available for the smb2
+ * file descriptor.
+ *
+ * Behave like smb2_service() with some differences:
+ *  - must be called with a fd returned by smb2_get_fd() or smb2_get_fds(),
+ *  - passing -1 as fd will trigger a new connection attempt on the next
+ *  resolved address, cf. smb2_get_fds().
+ *
+ * Returns:
+ *  0 : Success
+ * <0 : Unrecoverable failure. At this point the context can no longer be
+ *      used and must be freed by calling smb2_destroy_context().
+ *
+ */
+int smb2_service_fd(struct smb2_context *smb2, int fd, int revents);
+
+/*
+ * Set the timeout in seconds after which a command will be aborted with
+ * SMB2_STATUS_IO_TIMEOUT.
+ * If you use timeouts with the async API you must make sure to call
+ * smb2_service() at least once every second.
+ *
+ * Default is 0: No timeout.
+ */
+void smb2_set_timeout(struct smb2_context *smb2, int seconds);
+
+/*
+ * Set which version of SMB to negotiate.
+ * Default is to let the server pick the version.
+ */
+enum smb2_negotiate_version {
+        SMB2_VERSION_ANY  = 0,
+        SMB2_VERSION_ANY2 = 2,
+        SMB2_VERSION_ANY3 = 3,
+        SMB2_VERSION_0202 = 0x0202,
+        SMB2_VERSION_0210 = 0x0210,
+        SMB2_VERSION_0300 = 0x0300,
+        SMB2_VERSION_0302 = 0x0302,
+        SMB2_VERSION_0311 = 0x0311
+};
+
+void smb2_set_version(struct smb2_context *smb2,
+                      enum smb2_negotiate_version version);
+
 /*
  * Set the security mode for the connection.
  * This is a combination of the flags SMB2_NEGOTIATE_SIGNING_ENABLED
@@ -137,6 +231,28 @@ int smb2_service(struct smb2_context *smb2, int revents);
  */
 void smb2_set_security_mode(struct smb2_context *smb2, uint16_t security_mode);
 
+/*
+ * Set whether smb3 encryption should be used or not.
+ * 0  : disable encryption. This is the default.
+ * !0 : enable encryption.
+ */
+void smb2_set_seal(struct smb2_context *smb2, int val);
+
+/*
+ * Set whether smb2 signing should be required or not
+ * 0  : do not require signing. This is the default.
+ * !0 : require signing.
+ */
+void smb2_set_sign(struct smb2_context *smb2, int val);
+
+/*
+ * Set authentication method.
+ * SMB2_SEC_UNDEFINED (use KRB if available or NTLM if not)
+ * SMB2_SEC_NTLMSSP
+ * SMB2_SEC_KRB5
+ */
+void smb2_set_authentication(struct smb2_context *smb2, int val);
+
 /*
  * Set the username that we will try to authenticate as.
  * Default is to try to authenticate as the current user.
@@ -158,6 +274,17 @@ void smb2_set_domain(struct smb2_context *smb2, const char *domain);
  */
 void smb2_set_workstation(struct smb2_context *smb2, const char *workstation);
 
+/*
+ * Sets the address to some user defined object. May be used to make
+ * additional context data available in the async callbacks.
+ */
+void smb2_set_opaque(struct smb2_context *smb2, void *opaque);
+
+/*
+ * Returns the opaque pointer set with smb2_set_opaque.
+ */
+void *smb2_get_opaque(struct smb2_context *smb2);
+
 
 /*
  * Returns the client_guid for this context.
@@ -290,7 +417,7 @@ struct smb2_pdu;
  * ...
  * *, smb2_queue_pdu(smb2, pdu);
  *
- * See libnfs.c and smb2-raw-stat-async.c for examples on how to use
+ * See libsmb2.c and smb2-raw-stat-async.c for examples on how to use
  * this interface.
  */
 void smb2_add_compound_pdu(struct smb2_context *smb2,
@@ -316,7 +443,7 @@ struct smb2dir;
  *          This structure is freed using smb2_closedir().
  * -errno : An error occured.
  *          Command_data is NULL.
- */       
+ */
 int smb2_opendir_async(struct smb2_context *smb2, const char *path,
                        smb2_command_cb cb, void *cb_data);
 
@@ -396,7 +523,7 @@ struct smb2fh;
  *          This structure is freed using smb2_close().
  * -errno : An error occured.
  *          Command_data is NULL.
- */       
+ */
 int smb2_open_async(struct smb2_context *smb2, const char *path, int flags,
                     smb2_command_cb cb, void *cb_data);
 
@@ -464,6 +591,20 @@ int smb2_fsync(struct smb2_context *smb2, struct smb2fh *fh);
 uint32_t smb2_get_max_read_size(struct smb2_context *smb2);
 uint32_t smb2_get_max_write_size(struct smb2_context *smb2);
 
+struct smb2_read_cb_data {
+        struct smb2fh *fh;
+        uint8_t *buf;
+        uint32_t count;
+        uint64_t offset;
+};
+
+struct smb2_write_cb_data {
+        struct smb2fh *fh;
+        const uint8_t *buf;
+        uint32_t count;
+        uint64_t offset;
+};
+
 /*
  * PREAD
  */
@@ -481,8 +622,10 @@ uint32_t smb2_get_max_write_size(struct smb2_context *smb2);
  *    >=0 : Number of bytes read.
  * -errno : An error occured.
  *
- * Command_data is always NULL.
- */       
+ * Command_data is struct smb2_read_cb_data, which holds the arguments
+ * that were given to smb2_pread_async.
+ * This structure is automatically freed.
+ */
 int smb2_pread_async(struct smb2_context *smb2, struct smb2fh *fh,
                      uint8_t *buf, uint32_t count, uint64_t offset,
                      smb2_command_cb cb, void *cb_data);
@@ -512,10 +655,12 @@ int smb2_pread(struct smb2_context *smb2, struct smb2fh *fh,
  *    >=0 : Number of bytes written.
  * -errno : An error occured.
  *
- * Command_data is always NULL.
- */       
+ * Command_data is struct smb2_write_cb_data, which holds the arguments
+ * that were given to smb2_pwrite_async.
+ * This structure is automatically freed.
+ */
 int smb2_pwrite_async(struct smb2_context *smb2, struct smb2fh *fh,
-                      uint8_t *buf, uint32_t count, uint64_t offset,
+                      const uint8_t *buf, uint32_t count, uint64_t offset,
                       smb2_command_cb cb, void *cb_data);
 
 /*
@@ -524,7 +669,7 @@ int smb2_pwrite_async(struct smb2_context *smb2, struct smb2fh *fh,
  * server supports.
  */
 int smb2_pwrite(struct smb2_context *smb2, struct smb2fh *fh,
-                uint8_t *buf, uint32_t count, uint64_t offset);
+                const uint8_t *buf, uint32_t count, uint64_t offset);
 
 /*
  * READ
@@ -541,7 +686,10 @@ int smb2_pwrite(struct smb2_context *smb2, struct smb2fh *fh,
  *    >=0 : Number of bytes read.
  * -errno : An error occured.
  *
- * Command_data is always NULL.
+ * Command_data is struct smb2_read_cb_data, which holds the arguments
+ * that were given to smb2_read_async. offset denotes the offset in the file
+ * at which the read took place.
+ * This structure is automatically freed.
  */
 int smb2_read_async(struct smb2_context *smb2, struct smb2fh *fh,
                     uint8_t *buf, uint32_t count,
@@ -568,23 +716,28 @@ int smb2_read(struct smb2_context *smb2, struct smb2fh *fh,
  *    >=0 : Number of bytes written.
  * -errno : An error occured.
  *
- * Command_data is always NULL.
+ * Command_data is struct smb2_write_cb_data, which holds the arguments
+ * that were given to smb2_write_async. offset denotes the offset in the file
+ * at which the write took place.
+ * This structure is automatically freed.
  */
 int smb2_write_async(struct smb2_context *smb2, struct smb2fh *fh,
-                     uint8_t *buf, uint32_t count,
+                     const uint8_t *buf, uint32_t count,
                      smb2_command_cb cb, void *cb_data);
 
 /*
  * Sync write()
  */
 int smb2_write(struct smb2_context *smb2, struct smb2fh *fh,
-               uint8_t *buf, uint32_t count);
+               const uint8_t *buf, uint32_t count);
 
 /*
  * Sync lseek()
  */
 /*
- * smb2_seek() never blocks, thus no async version is needed.
+ * smb2_seek() SEEK_SET and SEEK_CUR are fully supported.
+ * SEEK_END only returns the end-of-file from the original open.
+ * (it will not call fstat to discover the current file size and will not block)
  */
 int64_t smb2_lseek(struct smb2_context *smb2, struct smb2fh *fh,
                    int64_t offset, int whence, uint64_t *current_offset);
@@ -753,7 +906,7 @@ int smb2_rename_async(struct smb2_context *smb2, const char *oldpath,
  */
 int smb2_rename(struct smb2_context *smb2, const char *oldpath,
               const char *newpath);
-        
+
 /*
  * Async truncate()
  *
@@ -801,6 +954,29 @@ int smb2_ftruncate(struct smb2_context *smb2, struct smb2fh *fh,
                    uint64_t length);
 
 
+/*
+ * READLINK
+ */
+/*
+ * Async readlink()
+ *
+ * Returns
+ *  0     : The operation was initiated. The link content will be
+ *          reported through the callback function.
+ * -errno : There was an error. The callback function will not be invoked.
+ *
+ * When the callback is invoked, status indicates the result:
+ *      0 : Success. Command_data is the link content.
+ * -errno : An error occured.
+ */
+int smb2_readlink_async(struct smb2_context *smb2, const char *path,
+                        smb2_command_cb cb, void *cb_data);
+
+/*
+ * Sync readlink()
+ */
+int smb2_readlink(struct smb2_context *smb2, const char *path, char *buf, uint32_t bufsiz);
+
 /*
  * Async echo()
  *
@@ -825,69 +1001,14 @@ int smb2_echo_async(struct smb2_context *smb2,
  */
 int smb2_echo(struct smb2_context *smb2);
 
+/*
+ * Some symbols have moved over to a different header file to allow better
+ * separation between dcerpc and smb2, so we need to include this header
+ * here to retain compatibility for apps that depend on those symbols.
+ */
+#include <smb2/libsmb2-dcerpc-srvsvc.h>
+
 #ifdef __cplusplus
 }
 #endif
-
-/* Low 2 bits desctibe the type */
-#define SHARE_TYPE_DISKTREE  0
-#define SHARE_TYPE_PRINTQ    1
-#define SHARE_TYPE_DEVICE    2
-#define SHARE_TYPE_IPC       3
-
-#define SHARE_TYPE_TEMPORARY 0x40000000
-#define SHARE_TYPE_HIDDEN    0x80000000
-
-struct srvsvc_netshareinfo1 {
-        const char *name;
-        uint32_t type;
-	const char *comment;
-};
-
-struct srvsvc_netsharectr1 {
-        uint32_t count;
-        struct srvsvc_netshareinfo1 *array;
-};
-
-struct srvsvc_netsharectr {
-        uint32_t level;
-        union {
-                struct srvsvc_netsharectr1 ctr1;
-        };
-};
-
-struct srvsvc_netshareenumall_req {
-        const char *server;
-        uint32_t level;
-        struct srvsvc_netsharectr *ctr;
-        uint32_t max_buffer;
-        uint32_t resume_handle;
-};
-
-struct srvsvc_netshareenumall_rep {
-        uint32_t level;
-        struct srvsvc_netsharectr *ctr;
-        uint32_t total_entries;
-        uint32_t resume_handle;
-
-        uint32_t status;
-};
-
-/*
- * Async share_enum()
- * This function only works when connected to the IPC$ share.
- *
- * Returns
- *  0     : The operation was initiated. Result of the operation will be
- *          reported through the callback function.
- * -errno : There was an error. The callback function will not be invoked.
- *
- * When the callback is invoked, status indicates the result:
- *      0 : Success. Command_data is struct srvsvc_netshareenumall_rep *
- *          This pointer must be freed using smb2_free_data().
- * -errno : An error occured.
- */
-int smb2_share_enum_async(struct smb2_context *smb2,
-                          smb2_command_cb cb, void *cb_data);
-
 #endif /* !_LIBSMB2_H_ */
